@@ -13,30 +13,32 @@ from gym.wrappers.monitoring.video_recorder import VideoRecorder
 
 from agent_frame.agent_base import AgentBase
 
-logging.basicConfig(level=logging.INFO, format='[%(levelname)s]\t%(asctime)s\t\t %(message)s')
-LOG = logging.getLogger(__name__)
-LOG.setLevel(logging.INFO)
-
 CONFIG_FILE_NAME = 'config.json'
-PLOT_FILE_PATH_DEFAULT = "../output/plot.png"
+PLOT_FILE_PATH_DEFAULT = "./output/media/plot.png"
+SCORE_LOG_FIXED_LENGTH = 7
+STEPS_LOG_FIXED_LENGTH = 5
 
 render = False
 episodes = 1000
 plot_frequency = 10
 log_frequency = 1
 video_frequency = 100
-video_dir = "../output"
+video_dir = "./output/media"
+log_dir = "./output/log"
 plot_title = "Agent Score by Number of Episodes"
 plot_x_label = "Episode"
 plot_y_label = "Score"
 
+total_steps = None
+
 
 def load_config(**config):
-    global render, episodes, plot_frequency, log_frequency, video_frequency, video_dir
+    global render, episodes, plot_frequency, log_frequency, log_dir, video_frequency, video_dir
     render = config.get('render', render)
     episodes = config.get('episodes', episodes)
     plot_frequency = config.get('plot_frequency', plot_frequency)
     log_frequency = config.get('log_frequency', log_frequency)
+    log_dir = config.get('log_dir', log_dir)
     video_frequency = config.get('video_frequency', video_frequency)
     video_dir = config.get('video_dir', video_dir)
 
@@ -62,10 +64,12 @@ game_args_parser.add_argument('--plot_y_label', '--py', type=str,
                               help="Sets label for plot's y-axis")
 game_args_parser.add_argument('--log_frequency', '--lf', type=int,
                               help="Sets logging frequency, set to 0 to disable. Will log progress every n episodes")
+game_args_parser.add_argument('--log_dir', '--ld', type=str,
+                              help="Sets destination directory to save logs to")
 game_args_parser.add_argument('--video_frequency', '--vf', type=int,
                               help="Video recording frequency, records every n episodes")
 game_args_parser.add_argument('--video_dir', '--vd', type=str,
-                              help="Sets destination directory to save video recordings too")
+                              help="Sets destination directory to save video recordings to")
 
 game_args = game_args_parser.parse_known_args()[0]
 overrides = {k: v for k, v in vars(game_args_parser.parse_known_args()[0]).items() if v is not None
@@ -73,6 +77,24 @@ overrides = {k: v for k, v in vars(game_args_parser.parse_known_args()[0]).items
 
 load_config(**overrides)
 
+if not os.path.exists(video_dir):
+    os.makedirs(video_dir)
+
+if not os.path.exists(log_dir):
+    os.makedirs(log_dir)
+
+logging.basicConfig(
+    level=logging.INFO,
+    format='[%(levelname)s]\t%(asctime)s\t\t %(message)s',
+    handlers=[
+        logging.FileHandler(log_dir + "/log.txt"),
+        logging.StreamHandler()
+    ]
+)
+
+LOG = logging.getLogger(__name__)
+LOG.setLevel(logging.INFO)
+
 
 def plot(episodes, scores):
     df = pd.DataFrame({
@@ -99,6 +121,23 @@ def determine_extension(env):
     return ".mp4"
 
 
+def format_as_fixed_length(value, fixed_length):
+    value = str(value)
+    return value + (fixed_length - len(value)) * " "
+
+
+def log_step_summary(episodes, current_episode, score, steps):
+    global total_steps
+
+    current_episode = format_as_fixed_length(current_episode, len(str(episodes)))
+    score = format_as_fixed_length(score, SCORE_LOG_FIXED_LENGTH)
+    steps = format_as_fixed_length(steps, STEPS_LOG_FIXED_LENGTH)
+
+    LOG.info(
+        "Episode: {} SCORE: {} EPISODE STEPS: {} TOTAL_STEPS: {}".format(current_episode, score,
+                                                                         steps, total_steps))
+
+
 def start(env, agent: AgentBase):
     global video_recorder
     scores = []
@@ -119,9 +158,7 @@ def start(env, agent: AgentBase):
         total_steps += steps
 
         if (episode + 1) % log_frequency == 0:
-            LOG.info(
-                "Episode: {} SCORE: {} STEPS: {} TOTAL_STEPS: {}".format(episode, score,
-                                                                         steps, total_steps))
+            log_step_summary(episodes, episode, score, steps)
 
         if episode % plot_frequency == 0:
             plot([i for i in range(episode)], scores)
@@ -133,12 +170,18 @@ def start(env, agent: AgentBase):
 
 
 def run_episode(env, agent, video_recorder=None):
+    global total_steps, render
+
     done = False
     state = env.reset()
     score = 0
-    steps = 1
+    steps = 0
     while not done:
-        agent.before()
+        agent.before(recording=video_recorder is not None,
+                     rendering=render,
+                     state=state,
+                     total_steps=total_steps,
+                     current_steps=steps)
 
         action = agent.act(state)
         next_state, reward, done, info = env.step(action)
@@ -156,7 +199,18 @@ def run_episode(env, agent, video_recorder=None):
         score += reward
 
         steps += 1
-        agent.after()
+        total_steps += 1
+
+        agent.after(recording=video_recorder is not None,
+                    rendering=render,
+                    state=state,
+                    next_state=next_state,
+                    reward=reward,
+                    total_steps=total_steps,
+                    current_steps=steps,
+                    done=done,
+                    score=score,
+                    info=info)
 
     return score, steps
 
@@ -178,6 +232,9 @@ def resolve_env(gym_env):
 
 
 def run(gym_env, agent: AgentBase):
+    global total_steps
+
+    total_steps = 0
     init_output_dir()
     env = resolve_env(gym_env)
     start(env, agent)
